package fr.eurecom.dsg.mapreduce;

import java.io.IOException;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.conf.Configured;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.TextInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.TextOutputFormat;
import org.apache.hadoop.util.Tool;
import org.apache.hadoop.util.ToolRunner;

public class PageRank extends Configured implements Tool {
	
	static class PageRankMapper extends Mapper<LongWritable, Text, Text, Text> {
    
		private Text outKey = new Text();	//output key
		private Text outValue = new Text();	//output value
		private String pagerankType = "P";	//identifier of the pagerank value
		private String neighListType = "N";	//identifier of the node structure

		@Override
        protected void map(LongWritable key, Text value, Context context) 
        		throws IOException, InterruptedException {
        	String line = value.toString();		//convert Text to string
       	 	String[] elements = line.split("\\s+");		//deconstruct the string into values

            String currentNode = elements[0];
            String pagerank = elements[1]; 
            String numNeigh = elements.length - 2;
            outValue.set(pagerankType + "\t" + pagerank + "\t" + currentNode + "\t" + numNeigh);
                
            String neighList = "";
            for (int i = 2; i < elements.length; i++) { 
            	outKey.set(elements[i]);
                context.write(outKey, outValue);
                neighList = neighList + "\t" + elements[i];
            }
            context.write(new Text(currentNode), new Text(neighListType + neighList));
		}
	}

	static class PageRankReducer extends Reducer<Text, Text, Text, Text> {

		private Text outputValue = new Text();
		private String pagerankType = "P";	//identifier of the pagerank value
		private String nodeType = "N";	//identifier of the node structure

		@Override
		protected void reduce(Text key, Iterable<Text> values, Context context) 
				throws IOException, InterruptedException {
        	double sum = 0;
            String outlinks = "";
			for (Text val : values) {
            	String temp = val.toString();           
                if (temp.startsWith(nodeType)) {
                	outlinks = temp.substring(temp.indexOf("\t")+1, temp.length());
                } else {
                	String[] array = temp.split("\t");
                    sum += Double.valueOf(array[1])/Double.valueOf(array[3]);
                }
            }
            outputValue.set(String.valueOf(sum) + outlinks);
            context.write(key, outputValue);
		}
	}


	public static void deleteDir(File dir) 
			throws IOException {
        if (!dir.isDirectory()) {
        	return ;
        }

    	File[] files = dir.listFiles();
        for (int i = 0; i < files.length; i++) {
        	File file = files[i];
            if (file.isDirectory()) {
            	deleteDir(file);
        	} else {
                boolean deleted = file.delete();
                if (!deleted) {
                    throw new IOException("Unable to delete file" + file);
                }
            }
        }
        dir.delete();
    }
        
        
	public static void main(String[] args) throws Exception {
		Configuration conf;
		Job job;
		int depth = 0; // variable to keep track of the recursion depth
		Path in,out;
  
		int ITERATIONS = Integer.parseInt(args[1]);
		depth++;
		while (depth <= ITERATIONS) {
			// reuse the conf reference with a fresh object
			conf = new Configuration();
			// set the depth into the configuration
			conf.set("recursion.depth", depth + "");
			job = new Job(conf);
			job.setJobName("PageRank_" + depth);
 
			job.setMapperClass(PageRankMapper.class);
			job.setReducerClass(PageRankReducer.class);
			job.setJarByClass(PageRank.class);
			// always work on the path of the previous depth
			if (depth == 1)
				in = new Path(args[0] + "/");
			else in = new Path(args[0]+"/depth_" + (depth - 1) + "/");
			out = new Path(args[0]+"/depth_" + depth);
      
			FileInputFormat.addInputPath(job, in);
			FileOutputFormat.setOutputPath(job, out);
			job.setInputFormatClass(TextInputFormat.class);
			job.setOutputFormatClass(TextOutputFormat.class);
			job.setOutputKeyClass(Text.class);
			job.setOutputValueClass(Text.class);
			// wait for completion and update the counter
			job.waitForCompletion(true);
			// delete the outputpath if already exists
			if(depth > 1)
				deleteDir(new File(args[0]+"/depth_"+(depth-1)));
			depth++;
		} 
	}

}

